From 927d908967181aea5c884d7d027cc8cabcfdc2e0 Mon Sep 17 00:00:00 2001
From: liuyingdong <liuyingdong@huawei.com>
Date: Tue, 27 Dec 2016 16:55:04 +0800
Subject: [PATCH] [patch]uvp-monitor need wait until stop_all_proc has finished
 during live-migration

---
 sys/dev/xen/control/control.c   |  2 ++
 sys/xen/xenstore/xenstore.c     | 33 +++++++++++++++++++++++++++++++++
 sys/xen/xenstore/xenstore_dev.c |  9 +++++++++
 sys/xen/xenstore/xenstorevar.h  |  4 ++++
 4 files changed, 48 insertions(+)

diff --git a/sys/dev/xen/control/control.c b/sys/dev/xen/control/control.c
index b6b3535..a921c19 100644
--- a/sys/dev/xen/control/control.c
+++ b/sys/dev/xen/control/control.c
@@ -346,7 +346,9 @@ xctrl_suspend()
 	int suspend_cancelled;
 
 	EVENTHANDLER_INVOKE(power_suspend_early);
+	xs_lock();
 	stop_all_proc();
+	xs_unlock();
 	EVENTHANDLER_INVOKE(power_suspend);
 
 	if (smp_started) {
diff --git a/sys/xen/xenstore/xenstore.c b/sys/xen/xenstore/xenstore.c
index 601e07d..6fea70d 100644
--- a/sys/xen/xenstore/xenstore.c
+++ b/sys/xen/xenstore/xenstore.c
@@ -1203,11 +1203,43 @@ xs_suspend(device_t dev)
 
 	sx_xlock(&xs.suspend_mutex);
 	sx_xlock(&xs.request_mutex);
 
 	return (0);
 }
 
 /**
+  *sx_try_slock() will return 0 if the shared/exclusive
+  *lock cannot be acquired immediately; otherwise the shared/exclusive lock
+  *will be acquired and a non-zero value will be returned.
+  */
+int
+xs_try_lock(void)
+{
+	return sx_try_xlock(&xs.request_mutex);
+}
+
+/**
+ xs_lock before stop_all_proc
+ */
+void
+xs_lock(void)
+{
+	sx_xlock(&xs.request_mutex);
+	return;
+}
+
+/**
+ xs_unlock after stop_all_proc
+ */
+void
+xs_unlock(void)
+{
+	sx_xunlock(&xs.request_mutex);
+	return;
+}
+
+/**
  * Resume XenStore operations after this VM is resumed.
  */
 static int
@@ -1218,6 +1250,7 @@ xs_resume(device_t dev __unused)
 
 	xs_init_comms();
 
 	sx_xunlock(&xs.request_mutex);
 
 	/*
diff --git a/sys/xen/xenstore/xenstore_dev.c b/sys/xen/xenstore/xenstore_dev.c
index 9c35178..4c99820 100644
--- a/sys/xen/xenstore/xenstore_dev.c
+++ b/sys/xen/xenstore/xenstore_dev.c
@@ -127,6 +127,15 @@ xs_dev_write(struct cdev *dev, struct uio *uio, int ioflag)
 	if (error != 0)
 		return (error);
 
+	/*wait until stop_all_proc has finished during live-migration*/
+	while(!xs_try_lock())
+	{
+		error = tsleep(u, PCATCH, "xsdwrite", hz/10);
+		if (error && error != EWOULDBLOCK)
+			return (error);
+	}
+	xs_unlock();
+
 	if ((len + u->len) > sizeof(u->u.buffer))
 		return (EINVAL);
 
diff --git a/sys/xen/xenstore/xenstorevar.h b/sys/xen/xenstore/xenstorevar.h
index fd3f0c1..6b09959 100644
--- a/sys/xen/xenstore/xenstorevar.h
+++ b/sys/xen/xenstore/xenstorevar.h
@@ -338,4 +338,8 @@ void xs_unregister_watch(struct xs_watch *watch);
  */
 struct sbuf *xs_join(const char *, const char *);
 
+int xs_try_lock(void);
+void xs_lock(void);
+void xs_unlock(void);
+
 #endif /* _XEN_XENSTORE_XENSTOREVAR_H */
-- 
2.8.3.windows.1

